'use strict';

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = wizard;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _neonBridge = require('neon-bridge');

var _neonBridge2 = _interopRequireDefault(_neonBridge);

var _gitConfig = require('git-config');

var _gitConfig2 = _interopRequireDefault(_gitConfig);

var _validateNpmPackageLicense = require('validate-npm-package-license');

var _validateNpmPackageLicense2 = _interopRequireDefault(_validateNpmPackageLicense);

var _validateNpmPackageName = require('validate-npm-package-name');

var _validateNpmPackageName2 = _interopRequireDefault(_validateNpmPackageName);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ROOT_DIR = _path2.default.resolve(__dirname, "..");
var TEMPLATES_DIR = _path2.default.resolve(ROOT_DIR, "templates");

var NEON_CLI_VERSION = JSON.parse(_fs2.default.readFileSync(_path2.default.resolve(ROOT_DIR, "package.json"), 'utf8')).version;
var NEON_BRIDGE_VERSION = _neonBridge2.default.version;

function compile(filename) {
  return _handlebars2.default.compile(_fs2.default.readFileSync(_path2.default.resolve(TEMPLATES_DIR, filename), 'utf8'), { noEscape: true });
}

var GITIGNORE_TEMPLATE = compile(".gitignore.hbs");
var CARGO_TEMPLATE = compile("Cargo.toml.hbs");
var NPM_TEMPLATE = compile("package.json.hbs");
var INDEXJS_TEMPLATE = compile("index.js.hbs");
var LIBRS_TEMPLATE = compile("lib.rs.hbs");
var README_TEMPLATE = compile("README.md.hbs");
var CPP_TEMPLATE = compile("binding.cc.hbs");
var GYP_TEMPLATE = compile("binding.gyp.hbs");

function die(err) {
  console.log(err);
  process.exit(1);
}

function guessAuthor() {
  try {
    var config = _gitConfig2.default.sync();
    if (config.user.name) {
      return {
        author: config.user.name,
        email: config.user.email
      };
    }
  } catch (e) {
    return {
      author: process.env.USER || process.env.USERNAME,
      email: undefined
    };
  }
}

var style = {
  project: _chalk2.default.cyan.bold,
  command: _chalk2.default.green.bold,
  path: _chalk2.default.cyan
};

function wizard(pwd, name, toolchain) {
  var its = (0, _validateNpmPackageName2.default)(name);
  if (!its.validForNewPackages) {
    var errors = (its.errors || []).concat(its.warnings || []);
    console.log("Sorry, " + errors.join(" and ") + ".");
    process.exit(1);
  }

  // check for a scoped name
  var scoped = name.match(/@([^\/]+)\/(.*)/);

  var _ref = scoped || [, null, name];

  var _ref2 = _slicedToArray(_ref, 3);

  var scope = _ref2[1];
  var local = _ref2[2];

  console.log("This utility will walk you through creating the " + style.project(name) + " Neon project.");
  console.log("It only covers the most common items, and tries to guess sensible defaults.");
  console.log();
  console.log("Press ^C at any time to quit.");

  var root = _path2.default.resolve(pwd, local);
  var guess = guessAuthor();

  _inquirer2.default.prompt([{
    type: 'input',
    name: 'version',
    message: "version",
    default: "0.1.0",
    validate: function validate(input) {
      if (_semver2.default.valid(input)) {
        return true;
      }
      return "Invalid version: " + input;
    }
  }, { type: 'input', name: 'description', message: "description" }, { type: 'input', name: 'node', message: "node entry point", default: "lib/index.js" }, { type: 'input', name: 'git', message: "git repository" }, { type: 'input', name: 'author', message: "author", default: guess.author }, { type: 'input', name: 'email', message: "email", default: guess.email }, {
    type: 'input',
    name: 'license',
    message: "license",
    default: "MIT",
    validate: function validate(input) {
      var its = (0, _validateNpmPackageLicense2.default)(input);
      if (its.validForNewPackages) {
        return true;
      }
      var errors = (its.errors || []).concat(its.warnings || []);
      return "Sorry, " + errors.join(" and ") + ".";
    }
  }], function (answers) {
    answers.name = {
      npm: {
        full: name,
        scope: scope,
        local: local
      },
      cargo: {
        external: local,
        internal: local.replace(/-/g, "_")
      }
    };
    var ctx = {
      project: answers,
      "neon-cli": {
        major: _semver2.default.major(NEON_CLI_VERSION),
        minor: _semver2.default.minor(NEON_CLI_VERSION),
        patch: _semver2.default.patch(NEON_CLI_VERSION)
      },
      "neon-bridge": {
        major: _semver2.default.major(NEON_BRIDGE_VERSION),
        minor: _semver2.default.minor(NEON_BRIDGE_VERSION),
        patch: _semver2.default.patch(NEON_BRIDGE_VERSION)
      },
      build: {
        cargo: {
          cmd: toolchain === 'default' ? [] : ["multirust", "run", toolchain]
        }
      }
    };

    var lib = _path2.default.resolve(root, _path2.default.dirname(answers.node));
    var src = _path2.default.resolve(root, "src");

    (0, _mkdirp2.default)(lib, function (err) {
      if (err) die(err);
      (0, _mkdirp2.default)(src, function (err) {
        if (err) die(err);
        _fs2.default.writeFileSync(_path2.default.resolve(root, ".gitignore"), GITIGNORE_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(root, "package.json"), NPM_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(root, "Cargo.toml"), CARGO_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(root, "README.md"), README_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(root, answers.node), INDEXJS_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(root, "binding.gyp"), GYP_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(src, "binding.cc"), CPP_TEMPLATE(ctx), { flag: 'wx' });
        _fs2.default.writeFileSync(_path2.default.resolve(src, "lib.rs"), LIBRS_TEMPLATE(ctx), { flag: 'wx' });

        var relativeRoot = _path2.default.relative(pwd, root);
        var relativeNode = _path2.default.relative(pwd, _path2.default.resolve(root, answers.node));
        var relativeRust = _path2.default.relative(pwd, _path2.default.resolve(root, src + "/lib.rs"));

        console.log();
        console.log("Woo-hoo! Your Neon project has been created in: " + style.path(relativeRoot));
        console.log();
        console.log("The main Node entry point is at: " + style.path(relativeNode));
        console.log("The main Rust entry point is at: " + style.path(relativeRust));
        console.log();
        console.log("To build your project, just run " + style.command("npm install") + " from within the " + style.path(relativeRoot) + " directory.");
        console.log("Then you can test it out with " + style.command("node -e 'require(\"./\")'") + ".");
        console.log();
        console.log("Happy hacking!");
      });
    });
  });
};